#!/bin/bash

ARG="$@"
SCRIPT=$(readlink -f "$0"); PID=$$
DIR=$(cd "$(dirname $SCRIPT)" && pwd)
HOMEDIR=$(getent passwd pi | cut -d: -f6)

temp_dir="$DIR/tmp"
log_file="$DIR/.logfile"
roms_dir="$HOMEDIR/RetroPie/roms/mame-libretro"
assets_path="$HOMEDIR/ADMIN/MENU/.assets"
cafca_dir="$HOMEDIR/CAFCA/data/mame-libretro"
download_dir="$HOMEDIR/downloads/MAME2003_Reference_Set_MAME0.78_ROMs_CHDs_Samples/roms"
torrent_file="$HOMEDIR/ADMIN/FILES/TORRENTS/mame078_romset.torrent"

prompt=$(sed -n '/^editor_prompt=/ {s/\(.*\)=//g;s/#.*$//;p}' "$assets_path/.settings")
preserve_roms=$(sed -n '/^editor_preserve_roms=/ {s/\(.*\)=//g;s/#.*$//;p}' "$assets_path/.settings")
finished=0

#________________________________________________________________________________

# SETUP

export NEWT_COLORS="$(cat $HOMEDIR/ADMIN/MENU/.assets/config/whiptail_theme.cfg)"

shopt -s expand_aliases
alias log="sudo tee -a \"$log_file\" &>/dev/null"

[ -d $temp_dir ] && sudo rm -R $temp_dir; mktemp -d --suffix=_TEMP --tmpdir=$DIR &>/dev/null && { mv -f $DIR/tmp*_TEMP $temp_dir; for tmp in $temp_dir; do sudo chmod 775 $temp_dir; sudo chown pi:root $temp_dir; done; }

sudo rm $download_dir/*.* 2>/dev/null

printf '' > "$temp_dir/downloaded"
printf '' > "$log_file"

clear

# -------------------------------------------------------------------------------

# MAIN

Main() {
  changed="$([ -f $temp_dir/.list ] && echo 1 || echo 0)"

  choice=$( \
    whiptail --menu "MENU" 0 0 0 \
      "APPEND" "" \
      "EDIT" "" \
      "REFRESH" "$([[ $changed -eq 1 ]] && echo ' *')" \
      "REMOVE" "" \
      "APPLY" "$([[ $changed -eq 1 ]] && echo ' *')" 2>&1 1>$(tty))

  [[ $? -gt 0 ]] && return 0

  case "$choice" in
    APPEND) 	Append 	;;
    EDIT) 	Edit 	;;
    REFRESH) 	Refresh ;;
    REMOVE) 	Remove 	;;
    APPLY) 	Apply 	;;
  esac
}

# -------------------------------------------------------------------------------

Append() {
  dls="$(cat $temp_dir/downloaded | wc -l)"

  clear

  choice=$( \
    whiptail --menu "MENU" 0 0 0 \
      "DOWNLOAD" "" \
      "SCRAPE" "$([[ $dls -gt 0 ]] && echo ' *')" \
      "MARQUEES" "" 2>&1 1>$(tty))

  [[ $? -gt 0 ]] && Main

  case "$choice" in
    DOWNLOAD) 	[[ "${global_funcs[@]}" =~ downloader ]] &&  downloader; Append	;;
    SCRAPE) 	[[ "${global_funcs[@]}" =~ scraper ]] 	 &&  scraper;    [ $? -eq 0 ] && Main || Append ;;
    MARQUEES) 	[[ "${global_funcs[@]}" =~ marquees ]]   &&  marquees;   [ $? -eq 0 ] && Main || Append ;;
  esac
}

# -------------------------------------------------------------------------------

Edit() { sleep 1; Main; }

# -------------------------------------------------------------------------------

Refresh() {
  clear
  [ ! -f "$temp_dir/.list" ] && cp -f "$assets_path/GAMES/.list" "$temp_dir/.list"

  # DECODE HTML CHARACHTERS
  sed -i 's/&apos;//' "$temp_dir/.list"
  perl -i -MHTML::Entities -pe 'decode_entities($_);' "$temp_dir/.list"

  # CHECK DUPLICATE LINES
  echo -e "\nDUPLICATE LINES:\n" >> "$log_file"
  while read row; do
    sed -i "/${row%% *}/d" "$temp_dir/.list"
    echo "$row" >> "$log_file"
  done< <(awk -F'\x27' 'seen[$12]++' "$temp_dir/.list")
  awk '{$1=""; print NR$0}' "$temp_dir/.list" | sudo tee "$temp_dir/.list" &>/dev/null

  # CHECK DUPLICATE TITLES
  while read row; do
    local title=$(awk -F'\x27' -v n="$row" 'NR==n { print $4" ("$2")" }' "$temp_dir/.list")
    sed -i "${row}{s/'[^']*'/'${title}'/2}" "$temp_dir/.list"
  done< <(awk -F'\x27' 'seen[$4]++' "$temp_dir/.list" | awk '{print $1}')

  # CHECK FOR MARQUEE AND SET FLAG
  for i in $(seq 1 $(cat "$temp_dir/.list" | wc -l)); do
    name=$(sed -n "${i}p" "$temp_dir/.list" | cut -d\' -f2)
    find "$assets_path/media/marquees" -type f -iname "$name*.ppm" -exec false {} +; found=$?;
    sed -i "${i}{s/'[^']*'/'${found}'/7}" "$temp_dir/.list"
  done

  update_lists
  Apply
}

# -------------------------------------------------------------------------------

Remove() {
  local entries=()

  for g in $(awk -F'\x27' '{print $2}' "$assets_path/GAMES/.list" | sort -k2,2 -t \'); do entries+=("$g" ""); done
  game=$(whiptail --menu "$(echo -e "\nSELECT GAME"):" --title "REMOVE" 0 0 0 "${entries[@]}" 2>&1 1>$(tty))

  [ $? -eq 0 ] && whiptail  --yesno "$(echo -e "\n  REMOVE GAME '$game' ?")" --title "! WARNING !" 0 0 2>&1 1>$(tty) || Main

  if [ $? -eq 0 ]; then
    for file in .list .shooters .spinners; do [ ! -f "$temp_dir/$file" ] && cp -f "$assets_path/GAMES/$file" "$temp_dir/$file"; sed -E -i "/${game}[' ]*/d" "$temp_dir/$file"; done
    for file in table timers clearram; do [ ! -f "$temp_dir/cafca_$file" ] && cp -f "$cafca_dir/$file" "$temp_dir/cafca_$file"; sed -E -i "/${game}[' ]*/d" "$temp_dir/cafca_$file"; done
    awk '{$1="";print NR""$0}' "$temp_dir/.list" | sudo tee "$temp_dir/.list" &>/dev/null
    whiptail  --msgbox "$(echo -e "\n  GAME '${game}' WAS REMOVED.")" --title "SUCCESS" 0 0 2>&1 1>$(tty)

    #[[ ! $preserve_roms -eq 1 ]] && [[ $(whiptail  --yesno "$(echo -e "\n  DELETE ROM '$game' ?")" --title "! WARNING !" 0 0 2>&1 1>$(tty); echo $?) -eq 0 ]] && sudo rm -f "$roms_dir/$game.zip"
  fi

  Main
}

# -------------------------------------------------------------------------------

Apply() {
  whiptail --yesno "APPLY CHANGES?" 0 0 2>&1 1>$(tty)

  if [ $? -eq 0 ]; then clear;
    for f in .list .shooters .spinners; do [ -f "$temp_dir/$f" ] && cp -vf "$temp_dir/$f" "$assets_path/GAMES/$f" | sed "s/.*${DIR##*\/}/'/"; done
    for f in table timers clearram; do [ -f "$temp_dir/cafca_$f" ] && cp -vf "$temp_dir/cafca_$f" "$cafca_dir/$f" | sed "s/.*${DIR##*\/}/'/"; done
  fi

  sudo rm -f $download_dir/*.* 2>/dev/null;
  find "$temp_dir/" -type f -print -exec rm -f {} \;
  printf '' > "$temp_dir/downloaded"
  #sudo rm -f "$temp_dir/.list" 2>/dev/null; printf '' > "$temp_dir/downloaded"
  Main
}

# ___________________________________________________________________________________________________________________

downloader() {
  local items=()
  local log=

  echo "Starting transmission.."

  sudo -u pi bash -c 'transmission-daemon' &>/dev/null; sleep 1

  declare -c id=$(sudo -u pi bash -c 'transmission-remote -l' | grep -A1 '^ID' | tail -n 1 | grep -m1 'MAME' | awk '{print $1}')
  [[ $id =~ ^[0-9]+$ ]] && sudo -u pi id=$id bash -c 'transmission-remote -t $id -r' &>/dev/null

  sudo -u pi file=$torrent_file bash -c 'transmission-remote -a $file; while [[ $(transmission-remote -l | grep -c Verifying) -gt 0 ]]; do :; done' &>/dev/null
  declare -c id=$(sudo -u pi bash -c 'transmission-remote -l' | grep -A1 '^ID' | tail -n 1 | grep -m1 'MAME' | awk '{print $1}')

  sudo -u pi id=$id bash -c 'transmission-remote -t $id -Gall' &>/dev/null
  sudo -u pi id=$id bash -c 'transmission-remote -t $id -f' | grep -we 'roms' | sed 's/:.*\/roms\// /' | sudo tee "$temp_dir/torrent.dump" &>/dev/null

  sed -i '/^.*.chd/d;/^.*.png/d' "$temp_dir/torrent.dump"

  local entries=($(awk '{ print $1"_"$2; }' "$temp_dir/torrent.dump")); for item in "${entries[@]}"; do items+=("${item%%_*}" "${item##*_}" OFF); done
  local indexes=$(whiptail --checklist "SELECT GAME" 0 0 0 "${items[@]}" 2>&1 1>$(tty)); indexes=($(echo "$indexes" | sed 's/\x22//g'))
  local file_list=$(IFS=,; echo "${indexes[*]}")
  local printed=0; local pipe="$log_file"

  if [[ ${#indexes[@]} -gt 0 ]]; then
    [ -d $output_dir ] && sudo rm $output_dir/* 2>/dev/null

    sudo -u pi id=$id list=$file_list bash -c 'transmission-remote -t $id -g$list; transmission-remote -t $id --start' &>/dev/null

    while true; do
      log=$(sudo -u pi id=$id bash -c 'transmission-remote -t $id -f' | awk '{if ($4 ~ "Yes") print;}');
      for p in $(echo "$log" | awk '{gsub(/%/,"", $2);print $2}'); do [[ $p -lt 100 ]] && { downloading=1; break; }; done

      clear; echo -e "Downloading:\n"

      [[ $printed -eq 0 ]] && printed=1 || pipe=/dev/null

      echo "$log" | awk 'NR==FNR{i=split($NF,x,/\/|\./);n=x[i-1];s[n]=$1"\t"$2"\t"n;l[n];next}{for (k in l) if ($2 == k){if(k in f == 0){$1="";$2="";gsub(/\x27/,"",$0);sub(/^\ +/,"",$0);s[k]=s[k]"\t\x27"$0"\x27"}f[k];next}}END{for(k in l)print s[k];}' - "$DIR/mame2003-index" | column -t | awk -F'\x27' 'BEGIN{FS=OFS="\x27"}{gsub(/\ +/, " ",$(NF-1)); print}' | sudo tee -a $pipe 2>&1; sleep 1
      #echo "$log" | awk 'NR==FNR{i=split($NF,x,/\/|\./);n=x[i-1];s[n]=$1"\t"$2"\t"n;l[n];next}{for (k in l) if ($2 == k){if(k in f == 0){$1="";$2="";sub(/^\ +/,"",$0);s[k]=s[k]"\t\x27"$0"\x27"}f[k];next}}END{for(k in l)print s[k];}' - "$DIR/mame2003-index" | column -t | awk -F'\x27' 'BEGIN{FS=OFS="\x27"}{gsub(/\ +/, " ",$(NF-1)); print}' | sudo tee -a $pipe 2>&1; sleep 1

      #echo "$log" | awk 'NR==FNR{ i=split($NF,x,/\/|\./); n=x[i-1]; s[n]=$1"\t"$2; l[n]; next;}{ for(k in l) if ($2 == k){if (k in f == 0){$2="";$1="";sub(/^\ +/,"",$0);print s[k]"\t"k"\t\x27"$0"\x27";} f[k]; next;}}' - "$DIR/mame2003-index"
      #echo "$log" | awk '{n=split($NF,a,"/"); print $1" "a[n]" "$2}' | column -t; sleep 1; sed -n "/${name}/ {s/[[:digit:]]\+\s${name}\s\+//g;p;q}" mame2003-index

      [[ $(echo "$log" | grep -wc '\s100%' ) -eq ${#indexes[@]} ]] && break
    done
  else
    return 1
  fi

  while [[ $(find $download_dir -type f -iname '*.part' | wc -l) -gt 0 ]]; do sleep 1; done; sleep 3

  find $download_dir -type f -iname '*.zip' ! -iname '*.part' -exec cp -v {} $roms_dir \; | sudo tee -a "$log_file" &>/dev/null
  for dir in $roms_dir; do sudo chown -R pi:pi $dir; sudo chmod -R 775 $dir; done

  for rom in $(find $download_dir -type f -iname '*.zip' ! -iname '*.part' -print | grep -o '[^/]*$'); do
    [ -f "$roms_dir/$rom" ] && echo "$roms_dir/$rom"
  done | sudo tee "$temp_dir/downloaded" &>/dev/null
}

# -------------------------------------------------------------------------------

scraper() {
  local SCRAPER='/opt/retropie/supplementary/skyscraper/Skyscraper'
  local WORKDIR="$temp_dir/scraper"
  local gamelist="$WORKDIR/gamelist.xml"
  local writing=0
  local count=0

  mkdir "$WORKDIR" &>/dev/null; sudo cp "$HOMEDIR/.emulationstation/gamelists/mame-libretro/gamelist.xml" "$WORKDIR/gamelist.xml"

  if [[ $(cat "$temp_dir/downloaded" | wc -l) -lt 1 ]]; then
    whiptail --yesno "SCRAPE ROMS DIR?" --title "NO DOWNLOADS" 0 0 0 2>&1 1>$(tty)
    [ $? -eq 0 ] && find $roms_dir -type f -iname '*.zip' -exec bash -c '[[ ! ${1} =~ ${0} ]] && echo "${0}"' {} "$(awk -F'\x27' '{print $12}' $temp_dir/.list)" \; | sudo tee "$temp_dir/downloaded" &>/dev/null
  fi

  if [[ $(cat "$temp_dir/downloaded" | wc -l) -gt 0 ]]; then

    while read line; do line=$(echo "$line" | sed -E "s/\x1B\[([0-9]{1,2}(;[0-9]{1,2})*)?[m,K,H,f,J]//gm" | sed -E "s/\x1B\([A-Z]{1}(\x1B\[[m,K,H,f,J])?//gm");
      #[[ "$line" =~ 'Pass 1' ]] && { writing=1; clear; }
      #[[ $writing -eq 1 ]] && { echo "$line"; (( count++ )); }
      [[ "$line" =~ 'Pass 1' ]] && { writing=1; clear; echo "" | sudo tee -a "$log_file"; }
      [[ $writing -eq 1 ]] && { (( count++ )); echo "$line" | sudo tee -a "$log_file"; }
      [[ $count -ge 15 ]] && { writing=0; count=0; }
    done< <( $SCRAPER -p mame-libretro -s screenscraper -i "$roms_dir" -g "$WORKDIR" -a "$HOMEDIR/.skyscraper/artwork.xml" --flags unattendskip --includefrom "$temp_dir/downloaded" ); echo -e "\nWRITING GAMELIST\n"

    $SCRAPER -p mame-libretro -i "$HOMEDIR/RetroPie/roms/mame-libretro" -g "$WORKDIR" -a "$HOMEDIR/.skyscraper/artwork.xml" --flags unattend &>/dev/null

    local game_count=$(grep -Ec '<game>' "$WORKDIR/gamelist.xml")

    for (( n=1; n<=$game_count; n++ )); do
      data=$(awk -v p="<game>" -v n="$n" '$0~p{i++}i==n' "$WORKDIR/gamelist.xml")
      path=$(echo "$data" | grep -E "<path>" | awk '{$1=$1;print}' | sed 's/<path>//g; s/<\/path>//g')
      name=$(echo "$path" | grep -o '[^/]*$' | cut -d. -f1)

      if [[ $(grep -c "$name" "$temp_dir/downloaded") -gt 0 ]] && [[ ${#data} -gt 0 ]]; then
        title=$(echo "$data" | grep -E "<name>" | awk '{$1=$1;print}' | sed 's/<name>//g; s/<\/name>//g')
        date=$(echo "$data" | grep -E "<releasedate>" | awk '{$1=$1;print}' | sed 's/<releasedate>//g; s/<\/releasedate>//g'); date="${date:0:4}"; [[ ${#date} -lt 1 ]] && date='19xx'
        publisher=$(echo "$data" | grep -E "<publisher>" | awk '{$1=$1;print}' | sed 's/<publisher>//g; s/<\/publisher>//g'); publisher=$(echo "$publisher" | cut -d, -f1); [[ ${#publisher} -lt 1 ]] && publisher='<unknown>'
        type='JOY'
        find "$assets_path/media/marquees" -type f -iname "$name*" -exec false {} +; marquee=$?
        echo "'$name' '$title' '$date' '$publisher' '$type' '$path' '$marquee' '[ ]'"

        [[ $(grep -c "<path>$path</path>" "$WORKDIR/gamelist.xml") -lt 1 ]] && echo "${data}" | sed 's/^/\t/' | sudo tee -a "$WORKDIR/gamelist.xml" &>/dev/null
      fi
    done | sudo tee "$WORKDIR/new_entries" &>/dev/null

    [ ! -f "$temp_dir/.list" ] && cp -f "$assets_path/GAMES/.list" "$temp_dir/.list"

    cat "$WORKDIR/new_entries" | awk '{print "0 "$0}' | sudo tee -a "$temp_dir/.list" &>/dev/null
    cat "$temp_dir/.list" | sort -k4,4 -t \' --output "$temp_dir/.list"; awk '{$1="";print NR""$0}' "$temp_dir/.list" | sudo tee "$temp_dir/.list" &>/dev/null
    printf '' > "$temp_dir/downloaded"

    whiptail  --msgbox "$(echo -e "FINISHED SCRAPING $(cat $WORKDIR/new_entries | wc -l) ROM(S)")" --title "SUCCESS" 0 0 2>&1 1>$(tty)
    return 0
  else
    whiptail  --msgbox "NOTHING TO SCRAPE.." --title "WARNING" 0 0 2>&1 1>$(tty)
    return 1
  fi
}

# ----------------------------------------------------------------------------------------------

update_lists() {
  echo -e "\nUPDATING LISTS..\n"

  # SHOOTERS
  echo -e "SHOOTERS:"
  [ ! -f "$temp_dir/.shooters" ] && sudo cp "$assets_path/GAMES/.shooters" "$temp_dir/.shooters"
  for game in $(awk -F'\x27' '{if (($10 ~ "GUN") || ($16 ~ "GUN")) print $2;}' "$temp_dir/.list"); do
    [[ $(grep -wc "^$game" "$temp_dir/.shooters") -eq 0 ]] && printf '%-8s\t1\n' "$game"
  done | sudo tee -a "$temp_dir/.shooters"
  for game in $(grep -v '^$\|^\s*#' "$temp_dir/.shooters" | awk '{print $1}'); do
    [[ $(grep -c "$game.zip" "$temp_dir/.list") -lt 1 ]] && sed -i "/^${game}\s.*$/d" "$temp_dir/.shooters"
  done
  sort -k1,1 "$temp_dir/.shooters" --output="$temp_dir/.shooters"

  # SPINNERS
  echo -e "SPINNERS:"
  [ ! -f "$temp_dir/.spinners" ] && sudo cp "$assets_path/GAMES/.spinners" "$temp_dir/.spinners"
  for game in $(awk -F'\x27' '{if (($10 ~ "SPIN") || ($16 ~ "SPIN")) print $2;}' "$temp_dir/.list"); do
    [[ $(grep -wc "^$game" "$temp_dir/.spinners") -eq 0 ]] && printf '%-8s\t72\t\t1\t\t255\t\t255\n' "$game"
  done | sudo tee -a "$temp_dir/.spinners"
  for game in $(grep -v '^$\|^\s*#' "$temp_dir/.spinners" | awk '{print $1}'); do
    [[ $(grep -c "$game.zip" "$temp_dir/.list") -lt 1 ]] && sed -i "/^${game}\s.*$/d" "$temp_dir/.spinners"
  done
  sort -k1,1 "$temp_dir/.spinners" --output="$temp_dir/.spinners"

  # CAFCA
  echo -e "\nCAFCA FILES:"
  for file in table timers clearram; do [ ! -f "$temp_dir/cafca_$file" ] && cp -f "$cafca_dir/$file" "$temp_dir/cafca_$file"; echo -e "\n${file^^}:"
    for game in $(awk -F'\x27' '{print $2}' "$temp_dir/.list"); do
      [[ $(grep -wc "^$game" "$temp_dir/cafca_$file") -eq 0 ]] && echo "$game $([[ $file == timers ]] && echo 1 || { [[ $file == clearram ]] && echo 0; })"
    done | sudo tee -a "$temp_dir/cafca_$file" #&>/dev/null
    sort -k 1,1 "$temp_dir/cafca_$file" | column -t | sudo tee "$temp_dir/cafca_$file" &>/dev/null
  done
}

# -------------------------------------------------------------------------------

marquees() {
  echo "MARQUEES"; sleep 1
  return 1
}

# ----------------------------------------------------------------------------------------------

declare -a global_funcs=()
while IFS=, read -ra globals; do for g in "${globals[@]}"; do [[ "$g" =~ ^'f '.* ]] && global_funcs+=("${g##* }"); done; done< <({ declare -p; declare -F; } | grep '^declare' | grep -Ev '\s[A-Z_]+' | sed 's/[=].*$//; s/^declare\s-//g; s/^ //; s/$/,/')

cleanup() { sudo /etc/init.d/transmission-daemon stop; sudo pkill -f "transmission-daemon"; printf '' > "$temp_dir/downloaded"; exit 0; }
trap 'cleanup &>/dev/null' EXIT INT HUP SIGINT SIGHUP TERM

Main

if [[ $(cat "$temp_dir/downloaded" | wc -l) -gt 0 ]] && [[ ! -f "$temp_dir/.list" ]]; then
  whiptail --yesno "$(cat tmp/downloaded | grep -o '[^/]*$')\n" --title "DELETE ROMS?" 0 0 0 2>&1 1>$(tty)
  [ $? -eq 0 ] && { clear; for rom in $(cat "$temp_dir/downloaded"); do sudo rm -vf "$rom"; done | sudo tee "$log_file"; }
fi

exit 0

# ----------------------------------------------------------------------------------------------------------------------------------------------

updater() {
  local name="${1}"
  declare -a pages=($(sed -n '/\[ \([^]]*\) \]/ { s/.*\[ \([^]]*\) \].*/\1/; s/ /\n/g;/^$/d; p }' "$assets_path/GAMES/.list" | sort -u))
  declare -a game_pages=($(grep -w -m1 "$name" "$assets_path/GAMES/.list" | awk -F'\x27' '{print $(NF-1)}' | sed 's/[][]//g;s/ //'))
  local inputType=$(grep "$name" "$assets_path/GAMES/.list" | awk -F'\x27' '{print $10}')

  action=$(whiptail --radiolist "ACTION" 0 0 0 "PAGES" "" ON "INPUT" "" OFF 2>&1 1>$(tty))

  if [[ $? -eq 0 && ${#action} -gt 0 ]]; then
    if [[ $action == PAGES ]]; then items=();
      for p in ${pages[@]}; do [[ "${game_pages[@]}" =~ "$p" ]] && items+=("$p" "" ON) || items+=("$p" "" OFF); done
        filter=$(whiptail --checklist "TOGGLE PAGES" 0 0 0 "${items[@]}" 2>&1 1>$(tty))
        if [ $? -eq 0 ] && [[ ${#filter} -gt 0 ]]; then
          filter=$(echo "$filter" | sed 's/\x22//g')
          #sudo cp "$assets_path/GAMES/.list" "$assets_path/GAMES/.list.bak$(date +%y%m%d%H%M%S)"
          sed -i "/${name}/ s/\[[^]]*\]/[ ${filter} ]/" "$assets_path/GAMES/.list"

          declare -a game_pages=($(grep -w -m1 "$name" "$assets_path/GAMES/.list" | awk -F'\x27' '{print $(NF-1)}' | sed 's/[][]//g;s/ //'))

          local inputType=$(grep "$name" "$assets_path/GAMES/.list" | awk -F'\x27' '{print $10}')
          local inShooters=$(grep -c "^$name" "$assets_path/GAMES/.shooters")
          local inSpinners=$(grep -c "^$name" "$assets_path/GAMES/.spinners")

          if [[ "${game_pages[@]}" =~ GUN ]] || [[ $inputType == GUN ]]; then
            [[ $inShooters -lt 1 ]] && { printf '%-8s\t1\n' "$name"; sort -k1,1 "$assets_path/GAMES/.shooters" --output="$assets_path/GAMES/.shooters"; }
          else
            [[ $inShooters -gt 0 ]] && { sudo sed -i "/^${name}/d" "$assets_path/GAMES/.shooters"; }
          fi

          if [[ "${game_pages[@]}" =~ SPIN ]] || [[ $inputType == SPIN ]]; then
            [[ $inSpinners -lt 1 ]] && { printf '%-8s\t72\t\t1\t\t255\t\t255\n' "$name"; sort -k1,1 "$assets_path/GAMES/.spinners" --output="$assets_path/GAMES/.spinners"; }
          else
            [[ $inSpinners -gt 0 ]] && { sudo sed -i "/^${name}/d" "$assets_path/GAMES/.spinners"; }
          fi
        fi
    elif [[ $action == INPUT ]]; then items=();
      declare -a types=($(awk -F'\x27' '{print $10}' "$assets_path/GAMES/.list" | sort -u))
      for type in ${types[@]}; do [[ "$type" == "$inputType" ]] && items+=("$type" "" ON) || items+=("$type" "" OFF); done

      new_type=$(whiptail --radiolist "SET TYPE" 0 0 0 "${items[@]}" 2>&1 1>$(tty))

      if [ $? -eq 0 ]; then
        sudo sed -i "/${name}/ s/\(\x27${inputType}\x27\)/\x27${new_type}\x27/" "$assets_path/GAMES/.list"

        local inShooters=$(grep -c "^$name" "$assets_path/GAMES/.shooters")
        local inSpinners=$(grep -c "^$name" "$assets_path/GAMES/.spinners")

        if [[ $new_type == GUN ]]; then
          [[ $inShooters -lt 1 ]] && { printf '%-8s\t1\n' "$name"; sort -k1,1 "$assets_path/GAMES/.shooters" --output="$assets_path/GAMES/.shooters"; }
        else
          [[ $inShooters -gt 0 ]] && { sudo sed -i "/^${name}/d" "$assets_path/GAMES/.shooters"; }
          [[ "${game_pages[@]}" =~ GUN ]] && { new_pages=$(echo "${game_pages[@]}" | sed 's/GUN//;s/ //'); sed -i "/${name}/ s/\[[^]]*\]/[ ${new_pages} ]/" "$assets_path/GAMES/.list"; }
        fi

        if [[ $new_type == SPIN ]]; then
          [[ $inSpinners -lt 1 ]] && { printf '%-8s\t72\t\t1\t\t255\t\t255\n' "$name"; sort -k1,1 "$assets_path/GAMES/.spinners" --output="$assets_path/GAMES/.spinners"; }
        else
          [[ $inSpinners -gt 0 ]] && { sudo sed -i "/^${name}/d" "$assets_path/GAMES/.spinners"; }
          [[ "${game_pages[@]}" =~ SPIN ]] && { new_pages=$(echo "${game_pages[@]}" | sed 's/SPIN//;s/ //'); sed -i "/${name}/ s/\[[^]]*\]/[ ${new_pages} ]/" "$assets_path/GAMES/.list"; }
        fi
      fi
    fi
  fi
}

# --------------------------------------------------------------------------------------
